package cz.zcu.fav.kiv.mjakubas.saf.command;

import cz.zcu.fav.kiv.mjakubas.saf.command.exception.CommandRegistrationException;
import cz.zcu.fav.kiv.mjakubas.saf.command.handler.*;
import cz.zcu.fav.kiv.mjakubas.saf.config.ConfigManager;
import cz.zcu.fav.kiv.mjakubas.saf.config.configs.RequestConfig;
import cz.zcu.fav.kiv.mjakubas.saf.parser.args.ArgsParser;
import cz.zcu.fav.kiv.mjakubas.saf.parser.tokenizer.Token;
import cz.zcu.fav.kiv.mjakubas.saf.parser.tokenizer.exception.TokenException;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.jetbrains.annotations.NotNull;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Properties;

/**
 * Class manages all application commands.
 */
public class CommandManager {
    private static final Logger L = LogManager.getLogger(CommandManager.class);

    private final CommandRegistration registration;
    private final ArgsParser argsParser;
    private RequestConfig config;

    /**
     * Instantiates new command manager with command 'localisation'.
     *
     * @param commandProperties command properties
     * @throws Exception if any exception happens.
     */
    public CommandManager(@NotNull Properties commandProperties) throws Exception {
        registration = new CommandRegistration(commandProperties);
        argsParser = new ArgsParser(commandProperties);
        config = ConfigManager.getManager().getRequestConfig();
    }

    /**
     * Registers all commands.
     *
     * @throws CommandRegistrationException if any command exception happens while registering commands.
     */
    public void registerAllCommandHandlers() throws CommandRegistrationException {
        try {
            registration.registerClassCommands(new AbsencesCommandsHandler(config));
            registration.registerClassCommands(new BuildingsCommandsHandler(config));
            registration.registerClassCommands(new ClassesCommandsHandler(config));
            registration.registerClassCommands(new EvaluationsCommandsHandler(config));
            registration.registerClassCommands(new RoomsCommandsHandler(config));
            registration.registerClassCommands(new ScheduleActionsCommandsHandler(config));
            registration.registerClassCommands(new ScheduleActionsDatesCommandsHandler(config));
            registration.registerClassCommands(new SchoolDaysCommandsHandler(config));
            registration.registerClassCommands(new SchoolHoursCommandsHandler(config));
            registration.registerClassCommands(new StudentsCommandsHandler(config));
            registration.registerClassCommands(new StudentsSubjectsCommandsHandler(config));
            registration.registerClassCommands(new SubjectListsCommandsHandler(config));
            registration.registerClassCommands(new SubjectsCommandsHandler(config));
            registration.registerClassCommands(new TeachersCommandsHandler(config));
        } catch (TokenException e) {
            L.error("Command handlers unsuccessfully registered");
            throw new CommandRegistrationException(e);
        }
        L.info("Command handlers successfully registered");
    }

    /**
     * Processes a command.
     *
     * @param args command
     * @throws Exception if any exception happens.
     */
    public void processCommand(String[] args) throws Exception {
        boolean processed = false;

        Map<Object, Map<String, List<Token>>> commandMap = registration.getCommandsMap();
        for (Map.Entry<Object, Map<String, List<Token>>> classEntry : commandMap.entrySet()) {
            for (Map.Entry<String, List<Token>> methodEntry : classEntry.getValue().entrySet()) {
                List<Class<?>> methodArgTypes = new ArrayList<>();
                List<Object> methodArgs = new ArrayList<>();

                if (argsParser.parseArgs(args, methodEntry.getValue(), methodArgTypes, methodArgs)) {
                    classEntry.getKey().getClass().getDeclaredMethod(methodEntry.getKey(),
                            methodArgTypes.toArray(new Class[0])).invoke(classEntry.getKey(), methodArgs.toArray());
                    processed = true;
                }
            }
        }

        if (!processed) {
            System.out.println("Command not found!");
        }
    }
}
