package cz.zcu.fav.kiv.mjakubas.saf.command;

import cz.zcu.fav.kiv.mjakubas.saf.parser.tokenizer.Token;
import cz.zcu.fav.kiv.mjakubas.saf.parser.tokenizer.Tokenizer;
import cz.zcu.fav.kiv.mjakubas.saf.parser.tokenizer.exception.TokenException;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.jetbrains.annotations.NotNull;

import java.lang.reflect.Method;
import java.util.*;

/**
 * Defines a method for registering all possible classes with command handlers.
 */
public class CommandRegistration {
    private static final Logger L = LogManager.getLogger(CommandRegistration.class);

    private final Map<Object, Map<String, List<Token>>> commandsMap = new HashMap<>();
    private final Tokenizer tokenizer;

    /**
     * Instantiates new registrator with {@link Properties} containing all possible command flags.
     *
     * @param properties {@link Properties} with all possible command flags.
     */
    public CommandRegistration(@NotNull Properties properties) {
        this.tokenizer = new Tokenizer(properties);
    }

    /**
     * Registers all handlers of a specific {@link Class}. Handler is recognized by {@link CommandHandler} annotation.
     *
     * @param commandsClass {@link Class} containing command handlers
     * @throws TokenException if command tokenization fails
     */
    public void registerClassCommands(@NotNull Object commandsClass) throws TokenException {
        Map<String, List<Token>> classCommandsMap = new HashMap<>();
        commandsMap.put(commandsClass, classCommandsMap);
        for (Method m : commandsClass.getClass().getDeclaredMethods()) {
            if (!m.isAnnotationPresent(CommandHandler.class)) {
                continue;
            }
            CommandHandler commandHandler = m.getAnnotation(CommandHandler.class);
            String[] args = commandHandler.value();
            L.trace(m.getName() + "has: " + Arrays.toString(args));
            Token[] tokens = tokenizer.tokenize(args).toArray(new Token[0]);
            L.trace("tokenized as: " + Arrays.toString(tokens));
            List<Token> tokenList = Arrays.stream(tokens).toList();

            classCommandsMap.put(m.getName(), tokenList);
        }
    }

    /**
     * Returs {@link Map} with all registered command handlers
     *
     * @return {@link Map} with all registered command handlers
     */
    public Map<Object, Map<String, List<Token>>> getCommandsMap() {
        return commandsMap;
    }
}
