package cz.zcu.fav.kiv.mjakubas.saf.command.handler;

import cz.zcu.fav.kiv.mjakubas.saf.command.CommandHandler;
import cz.zcu.fav.kiv.mjakubas.saf.command.exception.CommandException;
import cz.zcu.fav.kiv.mjakubas.saf.config.configs.RequestConfig;
import cz.zcu.fav.kiv.mjakubas.saf.entity.Absence;
import cz.zcu.fav.kiv.mjakubas.saf.entity.ScheduleActionDate;
import cz.zcu.fav.kiv.mjakubas.saf.entity.Student;
import cz.zcu.fav.kiv.mjakubas.saf.io.export.EntityExporter;
import cz.zcu.fav.kiv.mjakubas.saf.parser.filetype.FileType;
import cz.zcu.fav.kiv.mjakubas.saf.request.EntitiesRequests;
import org.jetbrains.annotations.NotNull;

public class AbsencesCommandsHandler {

    private final RequestConfig config;

    public AbsencesCommandsHandler(RequestConfig config) {
        this.config = config;
    }

    // region GET

    /**
     * Gets all {@link Absence} entities.
     *
     * @param fileType output {@link FileType} file type
     * @param filePath output file path
     * @throws CommandException if any exception occurs
     */
    @CommandHandler({"{query.absences}", "{httpMethod.get}",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetAll(@NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Absence[] absences = (Absence[]) EntitiesRequests.get(config,
                    RequestConfig.ABSENCE_ENTITIES_QUERY,
                    Absence.class).getDeserializedEntities();
            EntityExporter.export(absences, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.println("Absences successfully queried!");
    }

    /**
     * Gets {@link Absence} entity by id.
     *
     * @param id       entity id
     * @param fileType output {@link FileType} file type
     * @param filePath output file path
     * @throws CommandException if any exception occurs
     */
    @CommandHandler({"{query.absences}", "{httpMethod.get}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Absence[] absences = (Absence[]) EntitiesRequests.get(config,
                    RequestConfig.ABSENCE_ENTITIES_QUERY + RequestConfig.ID_QUERY + "/" + id,
                    Absence.class).getDeserializedEntities();
            EntityExporter.export(absences, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Absence with id: %d successfully queried!%n", id);
    }

    /**
     * Gets {@link Absence} entity by {@link Student} id.
     *
     * @param id       {@link Student} entity id
     * @param fileType output {@link FileType} file type
     * @param filePath output file path
     * @throws CommandException if any exception occurs
     */
    @CommandHandler({"{query.absences}", "{httpMethod.get}", "{query.sub.byStudent}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByStudentEntityID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Absence[] absences = (Absence[]) EntitiesRequests.get(config,
                    RequestConfig.ABSENCE_ENTITIES_QUERY + RequestConfig.STUDENT_QUERY + RequestConfig.ID_QUERY + "/" + id,
                    Absence.class).getDeserializedEntities();
            EntityExporter.export(absences, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Absence with student id: %d successfully queried!%n", id);
    }


    /**
     * Gets {@link Absence} entities by {@link Student} first and last name.
     *
     * @param firstName {@link Student} first name
     * @param lastName  {@link Student} last name
     * @param fileType  output {@link FileType} file type
     * @param filePath  output file path
     * @throws CommandException if any exception occurs
     */
    @CommandHandler({"{query.absences}", "{httpMethod.get}", "{query.sub.byStudent}",
            "{query.sub.byName}", "<java.lang.String>", "{query.sub.bySurname}", "<java.lang.String>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByStudentEntityFirstAndLastName(@NotNull String firstName, @NotNull String lastName,
                                                   @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Absence[] absences = (Absence[]) EntitiesRequests.get(config,
                    RequestConfig.ABSENCE_ENTITIES_QUERY + RequestConfig.STUDENT_QUERY + RequestConfig.NAME_QUERY +
                            "/" + firstName + "/" + lastName,
                    Absence.class).getDeserializedEntities();
            EntityExporter.export(absences, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Absence with student name: %s surname: %s successfully queried!%n", firstName, lastName);
    }

    /**
     * Gets {@link Absence} entities by {@link ScheduleActionDate} entity id.
     *
     * @param id       {@link ScheduleActionDate} entity id
     * @param fileType output {@link FileType} file type
     * @param filePath output file path
     * @throws CommandException if any exception occurs
     */
    @CommandHandler({"{query.absences}", "{httpMethod.get}", "{query.sub.byScheduleActionDate}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handleGetByScheduleActionDateEntityID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Absence[] absences = (Absence[]) EntitiesRequests.get(config,
                    RequestConfig.ABSENCE_ENTITIES_QUERY + RequestConfig.SCHEDULEACTIONDATE_QUERY + RequestConfig.ID_QUERY +
                            "/" + id,
                    Absence.class).getDeserializedEntities();
            EntityExporter.export(absences, fileType, filePath);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Absence with schedule action date id: %d successfully queried!%n", id);
    }
    // endregion

    // region POST

    /**
     * @param fileType
     * @param filePath
     * @throws CommandException
     */
    @CommandHandler({"{query.absences}", "{httpMethod.post}",
            "[query.sub.type]", "<java.lang.String>"})
    public void handlePost(@NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Absence[] absences = (Absence[]) EntityExporter.importToObjects(fileType, filePath, Absence.class);
            EntitiesRequests.post(config, RequestConfig.ABSENCE_ENTITIES_QUERY, absences);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Absence/Absences successfully created!%n");
    }
    // endregion

    // region PUT

    /**
     * Puts modified {@link Absence} entity by its id.
     *
     * @param id       {@link Absence} entity id
     * @param fileType output {@link FileType} file type
     * @param filePath output file path
     * @throws CommandException if any exception occurs
     */
    @CommandHandler({"{query.absences}", "{httpMethod.put}", "<java.lang.Integer>",
            "[query.sub.type]", "<java.lang.String>"})
    public void handlePutByID(Integer id, @NotNull FileType fileType, @NotNull String filePath)
            throws CommandException {
        try {
            Absence[] absences = (Absence[]) EntityExporter.importToObjects(fileType, filePath, Absence.class);
            EntitiesRequests.put(config, RequestConfig.ABSENCE_ENTITIES_QUERY + RequestConfig.ID_QUERY + "/" + id,
                    absences[0]);
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Absence with id: %d successfully modified!%n", id);
    }
    // endregion

    /**
     * Deletes {@link Absence} entity by its id.
     *
     * @param id {@link Absence} entity id
     * @throws CommandException if any exception occurs
     */
    // region DELETE
    @CommandHandler({"{query.absences}", "{httpMethod.delete}", "<java.lang.Integer>"})
    public void handleDeleteByID(Integer id)
            throws CommandException {
        String msg;
        try {
            msg = EntitiesRequests.delete(config,
                            RequestConfig.ABSENCE_ENTITIES_QUERY + RequestConfig.ID_QUERY + "/" + id).
                    getHttpResponseResult().getResponse();
        } catch (Exception e) {
            throw new CommandException(e);
        }
        System.out.printf("Absence with id: %d successfully deleted!%n", id);
    }
    // endregion
}
