package cz.zcu.fav.kiv.mjakubas.saf.gui.i18n;

import com.dlsc.formsfx.model.util.ResourceBundleService;
import javafx.beans.binding.Bindings;
import javafx.beans.binding.StringBinding;
import javafx.beans.property.ObjectProperty;
import javafx.beans.property.SimpleObjectProperty;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.jetbrains.annotations.NotNull;

import java.text.MessageFormat;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

/**
 * I18N utility class.
 */
public class I18N {

    private static final Logger L = LogManager.getLogger(I18N.class);
    private static final I18N INSTANCE = new I18N();
    private final ObjectProperty<Locale> locale;

    private I18N() {
        locale = new SimpleObjectProperty<>(new Locale("cs", "CZ"));
        locale.addListener(((observable, oldValue, newValue) -> Locale.setDefault(newValue)));
    }

    /**
     * Creates new {@link StringBinding} for given i18n key.
     *
     * @param key  value key
     * @param args opt
     * @return new {@link StringBinding}
     */
    public StringBinding createStringBinding(final String key, Object... args) {
        return Bindings.createStringBinding(() -> getValue(key, args), locale);
    }

    // get

    /**
     * Gets internationalization and localization tool class.
     *
     * @return internationalization and localization tool class
     */
    public static I18N getLocalizationTool() {
        return INSTANCE;
    }

    /**
     * Gets key value for current set internal {@link Locale}.
     *
     * @param key  value key
     * @param args opt
     * @return key value
     */
    public String getValue(@NotNull String key, Object... args) {
        ResourceBundle bundle = ResourceBundle.getBundle("frontend-locale", locale.get());
        String value = key;

        try {
            value = MessageFormat.format(bundle.getString(key), args);
        } catch (MissingResourceException e) {
            L.trace("Localisation key: " + key + " not found!");
            // ignored
        }

        return value;
    }

    /**
     * Gets i18n {@link ResourceBundle}.
     *
     * @return i18n {@link ResourceBundle}
     */
    public ResourceBundleService getResourceBundleService() {
        return new ResourceBundleService(ResourceBundle.getBundle("frontend-locale",
                new Locale(locale.getValue().getLanguage(), locale.getValue().getCountry())));
    }

    // set

    /**
     * Sets current {@link Locale} of i18n.
     *
     * @param locale current locale
     */
    public void setLocale(@NotNull Locale locale) {
        this.locale.set(locale);
        Locale.setDefault(locale);
    }
}
