package cz.zcu.fav.kiv.mjakubas.saf.io.export;

import cz.zcu.fav.kiv.mjakubas.saf.io.export.exception.ExportException;
import cz.zcu.fav.kiv.mjakubas.saf.parser.filetype.FileType;
import cz.zcu.fav.kiv.mjakubas.saf.parser.mapper.IMapper;
import cz.zcu.fav.kiv.mjakubas.saf.parser.mapper.JSONMapper;
import cz.zcu.fav.kiv.mjakubas.saf.parser.mapper.XMLMapper;
import cz.zcu.fav.kiv.mjakubas.saf.parser.mapper.exception.MapperException;
import org.jetbrains.annotations.NotNull;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;

/**
 * Class handles entity exports from/to file.
 */
public class EntityExporter {

    /**
     * Export entities to file.
     *
     * @param entities entities
     * @param fileType filetype
     * @param path     path
     * @throws ExportException if any exception happens.
     */
    public static void export(@NotNull Object[] entities, @NotNull FileType fileType, @NotNull String path) throws ExportException {
        switch (fileType) {
            case XML -> exportAsXML(entities, path);
            case JSON -> exportAsJSON(entities, path);
        }
    }

    public static void exportAsJSON(@NotNull Object[] entities, @NotNull String path) throws ExportException {
        try {
            IMapper mapper = new JSONMapper();
            exportToFile(mapper, entities, path);
        } catch (Exception e) {
            throw new ExportException(e);
        }
    }

    public static void exportAsXML(@NotNull Object[] entities, @NotNull String path) throws ExportException {
        try {
            IMapper mapper = new XMLMapper();
            exportToFile(mapper, entities, path);
        } catch (Exception e) {
            throw new ExportException(e);
        }
    }

    private static void exportToFile(IMapper mapper, @NotNull Object[] entities, @NotNull String path)
            throws MapperException, IOException {
        String serialization = mapper.serialize(entities);

        try (BufferedWriter writer = new BufferedWriter(new FileWriter(path))) {
            writer.write(serialization);
        }
    }

    /**
     * Imports entities from file as object.
     *
     * @param fileType filetype
     * @param path path
     * @param entityClass entities class
     * @return entities as objects
     * @throws ExportException if any exception happens
     */
    public static Object[] importToObjects(@NotNull FileType fileType, @NotNull String path, Class<?> entityClass) throws ExportException {
        switch (fileType) {
            case XML -> {
                return importToObjectsFromXML(path, entityClass);
            }
            case JSON -> {
                return importToObjectsFromJSON(path, entityClass);
            }
        }

        throw new ExportException("Filetype not found!");
    }


    public static Object[] importToObjectsFromXML(@NotNull String path, @NotNull Class<?> entityClass)
            throws ExportException {
        try {
            String result = Files.readString(Path.of(path));
            IMapper mapper = new XMLMapper();
            return mapper.deserialize(result, entityClass);
        } catch (Exception e) {
            throw new ExportException("Check file! IO error: " + path + "!");
        }
    }

    public static Object[] importToObjectsFromJSON(@NotNull String path, @NotNull Class<?> entityClass)
            throws ExportException {
        try {
            String result = Files.readString(Path.of(path));
            IMapper mapper = new JSONMapper();
            return mapper.deserialize(result, entityClass);
        } catch (Exception e) {
            throw new ExportException("Check file! IO error: " + path + "!");
        }
    }
}
