package cz.zcu.fav.kiv.mjakubas.saf.io.properties;

import cz.zcu.fav.kiv.mjakubas.saf.io.properties.exception.MissingPropertyKeyException;
import cz.zcu.fav.kiv.mjakubas.saf.parser.filetype.FileType;
import org.jetbrains.annotations.NotNull;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Properties;

/**
 * I/O methods working with '.properties' files loaded as {@link Properties} class.
 */
public class PropertiesIO {

    /**
     * Gets property value by its key if it exists. Otherwise, throws {@link IOException}.
     *
     * @param properties {@link Properties} class
     * @param key        property key
     * @return property value
     * @throws IOException if property key doesn't exist
     */
    public static String getValue(@NotNull Properties properties, @NotNull String key) throws IOException {
        String value = properties.getProperty(key);
        if (value == null) {
            throw new IOException("missing property value");
        }
        return value;
    }

    private static final String QUERY_SUB_TYPE = "query.sub.type.";

    /**
     * Gets property value of {@link FileType} command id it exists. Otherwise, throws {@link MissingPropertyKeyException}.
     *
     * @param properties {{@link Properties} class}
     * @param value      property value
     * @return property {@link FileType}
     * @throws MissingPropertyKeyException if property value doesn't exist
     */
    public static FileType getFileTypeValue(@NotNull Properties properties, @NotNull String value)
            throws MissingPropertyKeyException {
        for (FileType type : FileType.values()) {
            String typeKey = QUERY_SUB_TYPE + type.name();
            if (!properties.containsKey(typeKey)) {
                continue;
            }
            if (!properties.getProperty(typeKey).equalsIgnoreCase(value)) {
                continue;
            }
            return type;
        }

        throw new MissingPropertyKeyException("missing property key");
    }

    /**
     * Returns {@link Properties} instance representing loaded properties file.
     *
     * @param propertiesPath properties file path
     * @return {@link Properties} instance
     * @throws NullPointerException if any parameter is null
     * @throws IOException          in any I/O exception occurs
     */
    public static Properties getProperties(String propertiesPath) throws IOException {
        FileInputStream fileInputStream;
        try {
            fileInputStream = new FileInputStream(new File(propertiesPath));
        } catch (IOException e) {
            throw e;
        }

        Properties properties = new Properties();
        try {
            properties.load(fileInputStream);
        } catch (IOException e) {
            throw e;
        }

        return properties;
    }
}
