package cz.zcu.fav.kiv.mjakubas.saf.parser.tokenizer;

import cz.zcu.fav.kiv.mjakubas.saf.io.properties.PropertiesIO;
import cz.zcu.fav.kiv.mjakubas.saf.parser.filetype.FileType;
import cz.zcu.fav.kiv.mjakubas.saf.parser.tokenizer.exception.TokenException;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.jetbrains.annotations.NotNull;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

/**
 * Tokenizes command.
 */
public class Tokenizer {
    private static final Logger L = LogManager.getLogger(Tokenizer.class);

    public static final char CLASS_TOKEN_IDENTIFIER = '<';
    public static final char COMMAND_STRING_TOKEN_IDENTIFIER = '{';
    public static final char FILETYPE_CLASS_TOKEN_IDENTIFIER = '[';

    private final Properties commandProperties;

    /**
     * Instantiates new {@link Tokenizer} with command {@link Properties}.
     *
     * @param commandProperties {@link Properties} containing list of accepted commands
     */
    public Tokenizer(@NotNull Properties commandProperties) {
        this.commandProperties = commandProperties;
    }

    /**
     * Tokenizes array of {@link String} arguments.
     *
     * @param args array of {@link String} arguments
     * @return tokenized array of {@link String} arguments
     * @throws TokenException if any argument fails to be tokenized
     */
    public List<Token> tokenize(@NotNull String[] args) throws TokenException {
        List<Token> tokenList = new ArrayList<>();
        for (String arg : args) {
            try {
                if (isClassValue(arg)) {
                    tokenList.add(new Token(processClassValue(arg)));
                    continue;
                }
                if (isFileTypeValue(arg)) {
                    tokenList.add(new Token(processFileTypeValue()));
                    continue;
                }

                tokenList.add(new Token(processStringValue(arg)));
            } catch (Exception e) {
                throw new TokenException(e);
            }
        }

        return tokenList;
    }

    /**
     * Checks if argument is expected to be of {@link Class} value.
     *
     * @param arg argument
     * @return true or false
     */
    public boolean isClassValue(@NotNull String arg) {
        return arg.charAt(0) == CLASS_TOKEN_IDENTIFIER;
    }

    /**
     * Checks if argument is expected to be of {@link FileType} value.
     *
     * @param arg argument
     * @return true or false
     */
    public boolean isFileTypeValue(@NotNull String arg) {
        return arg.charAt(0) == FILETYPE_CLASS_TOKEN_IDENTIFIER;
    }

    /**
     * Tries to process a command argument as a {@link Class} value. The argument should be wrapped in angle brackets.
     *
     * @param arg command argument
     * @return {@link Class} of token
     * @throws ClassNotFoundException if {@link Class} is not found
     */
    public Class<?> processClassValue(@NotNull String arg) throws ClassNotFoundException {
        return Class.forName(arg.substring(1, arg.length() - 1));
    }

    /**
     * Processes a command as a {@link FileType}. The argument should be wrapped in square brackets.
     *
     * @return {@link FileType} {@link Class}
     */
    public Class<?> processFileTypeValue() {
        return FileType.class;
    }

    /**
     * Process a command as a {@link String} value either as a fixed command value or as an
     * expected command value wrapped in curly brackets.
     *
     * @param arg command argument
     * @return {@link String} value
     * @throws IOException if expected command doesn't exist
     */
    public String processStringValue(@NotNull String arg) throws IOException {
        if (arg.charAt(0) == COMMAND_STRING_TOKEN_IDENTIFIER) {
            return PropertiesIO.getValue(commandProperties, arg.substring(1, arg.length() - 1));
        }
        return arg;
    }
}
