package cz.zcu.fav.kiv.mjakubas.saf.request;

import cz.zcu.fav.kiv.mjakubas.saf.config.configs.RequestConfig;
import cz.zcu.fav.kiv.mjakubas.saf.parser.mapper.IMapper;
import cz.zcu.fav.kiv.mjakubas.saf.parser.mapper.JSONMapper;
import cz.zcu.fav.kiv.mjakubas.saf.parser.mapper.exception.MapperException;
import cz.zcu.fav.kiv.mjakubas.saf.request.basicrequest.HttpResponseResult;
import cz.zcu.fav.kiv.mjakubas.saf.request.basicrequest.exception.RequestErrorException;
import cz.zcu.fav.kiv.mjakubas.saf.request.basicrequest.requests.HttpDeleteRequest;
import cz.zcu.fav.kiv.mjakubas.saf.request.basicrequest.requests.HttpGetRequest;
import cz.zcu.fav.kiv.mjakubas.saf.request.basicrequest.requests.HttpPostRequest;
import cz.zcu.fav.kiv.mjakubas.saf.request.basicrequest.requests.HttpPutRequest;
import org.jetbrains.annotations.NotNull;

import java.net.URI;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * Entities HTTP request to back-end.
 */
public class EntitiesRequests {

    public static final int OK_RESULT = 200;


    /**
     * GET HTTP request. Gets all entities of certain type.
     *
     * @param config request config
     * @param path REST back-end path address
     * @param entity fetched entity class
     * @return fetched entities
     * @param <T> entity type class
     * @throws RequestErrorException if any exception happens
     */
    public static <T> EntitiesRequestResult get(@NotNull RequestConfig config, @NotNull String path, @NotNull Class<T> entity)
            throws RequestErrorException {
        path = path.replaceAll(" ", "%20");
        URI uri = URI.create(config.address() + path);
        HttpResponseResult response = new HttpGetRequest(uri, config.authenticator()).doRequest(null);
        if (response.getResponseCode() != OK_RESULT) {
            throw new RequestErrorException(response.getResponseCode() + ": " + response.getResponse());
        }
        /* fk */
        if (response.getResponse().contains("substitutions")) {
            response = new HttpResponseResult(response.getResponseCode(), "[]");
        }
        if (response.getResponse().contains("Average")) {
            return new EntitiesRequestResult(response, null);
        }
        if (response.getResponse().charAt(0) != '[') {
            response = new HttpResponseResult(response.getResponseCode(), "[" + response.getResponse() + "]");
        }
        /* fk */
        IMapper mapper = new JSONMapper();
        try {
            T[] deserialization = mapper.deserialize(response.getResponse(), entity);
            return new EntitiesRequestResult(response, deserialization);
        } catch (MapperException e) {
            throw new RequestErrorException(e);
        }
    }

    /**
     * POST HTTP request. Posts all entities of certain type.
     *
     * @param config request config
     * @param path REST back-end path address
     * @param entities post entities
     * @return result of posting entities
     *
     * @throws RequestErrorException if any exception happens
     */
    public static EntitiesRequestResult[] post(@NotNull RequestConfig config, @NotNull String path, @NotNull Object[] entities)
            throws RequestErrorException {
        EntitiesRequestResult[] results = new EntitiesRequestResult[entities.length];
        URI uri = URI.create(config.address() + path);
        IMapper mapper = new JSONMapper();
        int i = 0;
        for (Object entity : entities) {
            String serialization;
            try {
                serialization = mapper.serialize(entity);
            } catch (MapperException e) {
                throw new RequestErrorException(e);
            }
            HttpResponseResult response = new HttpPostRequest(uri, config.authenticator()).doRequest(serialization);
            if (response.getResponseCode() != OK_RESULT) {
                throw new RequestErrorException(response.getResponseCode() + ": " + response.getResponse());
            }
            results[i++] = new EntitiesRequestResult(response, null);
        }
        return results;
    }

    /**
     * MULTI POST HTTP request. Post multiple entities of different types.
     *
     * @param config request config
     * @param map map of all entities and their base post address
     * @return result of multi post
     *
     * @throws RequestErrorException if any exception happens
     */
    public static EntitiesRequestResult[] multiPost(@NotNull RequestConfig config,
                                                    Map<String, List<Object>> map)
            throws RequestErrorException {
        int len = 0;
        for (List<Object> os : map.values()) {
            len += os.size();
        }
        EntitiesRequestResult[] results = new EntitiesRequestResult[len];
        int l = 0;
        for (String key : map.keySet()) {
            Object[] os = map.get(key).toArray(new Object[0]);

            URI uri = URI.create(config.address() + key);
            IMapper mapper = new JSONMapper();
            for (Object entity : os) {
                String serialization;
                try {
                    serialization = mapper.serialize(entity);
                } catch (MapperException e) {
                    throw new RequestErrorException(e);
                }
                HttpResponseResult response = new HttpPostRequest(uri, config.authenticator()).doRequest(serialization);
                if (response.getResponseCode() != 200) {
                    throw new RequestErrorException(response.getResponseCode() + ": " + response.getResponse());
                }
                results[l++] = new EntitiesRequestResult(response, null);
            }
        }
        return results;
    }

    /**
     * PUT HTTP request. Put entity and modifies it in back-end.
     *
     * @param config request config
     * @param path put address
     * @param entity modified entities
     * @return put result
     *
     * @throws RequestErrorException if any exception happens
     */
    public static EntitiesRequestResult put(@NotNull RequestConfig config, @NotNull String path, @NotNull Object entity)
            throws RequestErrorException {
        HttpResponseResult response = null;
        try {
            URI uri = URI.create(config.address() + path);
            IMapper mapper = new JSONMapper();
            String serialization = mapper.serialize(entity);
            response = new HttpPutRequest(uri, config.authenticator()).doRequest(serialization);


        } catch (Exception e) {
            if (Objects.requireNonNull(response).getResponseCode() != OK_RESULT)
                throw new RequestErrorException(response.getResponse());
            throw new RequestErrorException(e);
        }

        if (Objects.requireNonNull(response).getResponseCode() != OK_RESULT)
            throw new RequestErrorException(response.getResponse());
        return new EntitiesRequestResult(response, null);
    }

    /**
     * DELETE HTTP request. Deletes entity from back-end.
     *
     * @param config request config
     * @param path delete REST path address
     * @return delete result
     *
     * @throws RequestErrorException if any exception happens
     */
    public static EntitiesRequestResult delete(@NotNull RequestConfig config, @NotNull String path)
            throws RequestErrorException {
        URI uri = URI.create(config.address() + path);
        HttpResponseResult response = new HttpDeleteRequest(uri, config.authenticator()).doRequest(null);
        if (response.getResponseCode() != OK_RESULT) {
            throw new RequestErrorException(response.getResponseCode() + ": " + response.getResponse());
        }
        return new EntitiesRequestResult(response, null);
    }
}
