package cz.zcu.fav.kiv.mjakubas.saf.request.basicrequest;

import cz.zcu.fav.kiv.mjakubas.saf.request.basicrequest.exception.RequestErrorException;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.jetbrains.annotations.NotNull;

import java.io.IOException;
import java.net.Authenticator;
import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;

/**
 * Represents a basic template for one http request method.
 * Allows to create a basic template for one http request that can be called continuously.
 */
public abstract class ABasicHttpRequest {
    private static final Logger L = LogManager.getLogger(ABasicHttpRequest.class);

    protected URI uri;
    protected Authenticator authenticator;

    /**
     * Creates a basic template for one http request. Request can be called continuously.
     *
     * @param uri           http request target uri
     * @param authenticator authentication method
     * @throws NullPointerException if any parameter is null
     */
    public ABasicHttpRequest(@NotNull URI uri, @NotNull Authenticator authenticator) throws NullPointerException {
        this.uri = uri;
        this.authenticator = authenticator;
    }

    /**
     * Sends a http request and returns its response.
     *
     * @param body http request body if needed
     * @return http request response
     * @throws RequestErrorException if error occurs during request
     */
    public HttpResponseResult doRequest(String body) throws RequestErrorException {
        HttpRequest request;
        try {
            request = buildRequest(body);
        } catch (NullPointerException e) {
            L.error(e);
            throw new RequestErrorException(e);
        }
        HttpResponse<String> response;
        try {
            response = HttpClient.newBuilder()
                    .authenticator(authenticator)
                    .build()
                    .send(request, HttpResponse.BodyHandlers.ofString());
        } catch (IOException | InterruptedException e) {
            throw new RequestErrorException("Connection error!");
        }
        return new HttpResponseResult(response.statusCode(), response.body());
    }

    /**
     * Request builder. Needs to be implemented precisely for one http method.
     *
     * @param body http request body if necessary
     * @return {@link HttpRequest}
     * @throws NullPointerException if body parameter is required but is null
     */
    protected abstract HttpRequest buildRequest(String body) throws NullPointerException;
}
